/***************************************************************************
****头文件************************************************************
***************************************************************************/

#include <Wire.h>//Arduino 官方I2C库
#include <SparkFun_APDS9960.h>//红外手势传感器库
#include <DFRobot_NeoPixel.h>//WS2812 RGB 全彩灯带库
/***************************************************************************
****数字引脚定义************************************************************
***************************************************************************/
#define IA1  10    //D6  Motor 1 PWM调速控制，范围0-255,0电机停转，255电机最大速度
#define IA2  12   //D5  Motor 2 PWM调速控制，范围0-255,0电机停转，255电机最大速度
#define IB1  11   //D7  Motor 1 转向控制，高电平正转，低电平反转
#define IB2  13   //D4  Motor 2 转向控制，高电平正转，低电平反转
#define PINS_APDS9960_INT 2 //D8  手势&颜色传感器中断引脚
#define PINS_wireLED  9   //D2  灯带

/***************************************************************************
****模拟引脚定义************************************************************
***************************************************************************/
// A4       SDA          I2C Data
// A5       SCL          I2C Clock

/***************************************************************************
****常数定义************************************************************
***************************************************************************/
#define True            1
#define False           0
////马达正反转定义
#define Positive_rotation   1
#define Reverse_rotation  0
//行进指令
#define Forward        1
#define Back           2
#define Turn_left      3
#define Turn_right     4
#define Stop           5
//手势动作
#define No_gesture     0
#define Down_gesture   4
#define Up_gesture     3
#define Left_gesture   1
#define Right_gesture  2
#define Far_gesture    6
#define Near_gesture   5
//灯带变色
#define RGB_up             False
#define RGB_down           True
#define Hue_up             False
#define Hue_down           True
#define Saturation_up             False
#define Saturation_down           True

//状态机
#define Ges_cycLED        1//手势识别，灯带循环渐变
#define Ges_staLED        2//手势识别，灯带静态显示
#define Ges_motionMEM     3//手势识别，记录手势到数组
#define Lit_detect        10//颜色识别
#define Pxm_detect        20//测量距离
#define Motion_busy       50//小灯正在做动作，不得打扰
/***************************************************************************
****全局变量************************************************************
***************************************************************************/
SparkFun_APDS9960 apds = SparkFun_APDS9960();
DFRobot_NeoPixel neoPixel_2;
int isr_flag = 0;//红外手势传感器外部中断
struct wireLED_color//灯带颜色
{
  uint32_t Red;
  uint16_t Green;
  uint16_t Blue;
};
struct ColorDetect//颜色识别
{
  uint16_t Red;
  uint16_t Green;
  uint16_t Blue;
  uint16_t Ambient;//亮度
};
struct ColorRGB_MAX
{
  uint32_t Red;
  uint16_t Green;
  uint16_t Blue;
};
struct ColorRGB_MIN
{
  uint32_t Red;
  uint16_t Green;
  uint16_t Blue;
};
static bool RGB_DIR;//灯带颜色渐变->色彩RGB变化方向
static wireLED_color LED_HSLChange;//距离传感器调节亮度
static uint8_t Ambient_mem;//亮度
static wireLED_color LED_RGB;
static wireLED_color LED_RGB_MEM;
static ColorRGB_MAX wireRGB_MAX;
static ColorRGB_MIN wireRGB_MIN;
static uint8_t Gesture_command;
static uint8_t Current_PROM_Mode;
ColorDetect Detected_Color;
static uint8_t MotionWithGeature[10];//按手势运动，存好最多十个手势，小灯按照手势运动
static uint8_t MWG_point;//手势记录数组的指针
struct Prom_Start_Time { //程序开始时的系统时间
  uint8_t Program_name;
  float Start_Time;
};
static Prom_Start_Time Prom_Start_Time;
/***************************************************************************
****子函数************************************************************
***************************************************************************/
void MA1_Forward(int Speed1) //正向快衰模式，Speed1值越大，电机转动越快
{
  analogWrite(IA1, Speed1);
  digitalWrite(IA2, LOW);
}

void MA2_Backward(int Speed1) //反向慢衰模式，Speed2值越小，电机转动越快
{
  int Speed2 = 255 - Speed1;
  analogWrite(IA1, Speed2);
  digitalWrite(IA2, HIGH);
}

void MB1_Forward(int Speed1)
{
  analogWrite(IB1, Speed1);
  digitalWrite(IB2, LOW);
}

void MB2_Backward(int Speed1)
{
  int Speed2 = 255 - Speed1;
  analogWrite(IB1, Speed2);
  digitalWrite(IB2, HIGH);
}
void WireLED_color_SET(wireLED_color *LED)
{
  neoPixel_2.setRangeColor(0, 9, LED->Red * 16 * 16 * 16 * 16
                           + LED->Green * 16 * 16
                           + LED->Blue );
}
void Car_action(uint8_t command)
{
  switch (command)
  {
    case Forward://Move ahead
      MA1_Forward(100);
      MB2_Backward(100);


      break;
    case Back://move back
      MA2_Backward(100);
      MB1_Forward(100);

      break;
    case Turn_left://turn left
      MA2_Backward(100);
      MB2_Backward(100);
      break;
    case Turn_right://turn right
      MA1_Forward(100);//You can change the speed, such as Motor(50,true)
      MB1_Forward(100);
      break;
    case Stop://stop
      MA2_Backward(0);
      MB2_Backward(0);
      break;
  }
}
void WireLED_RGBloop()
{
  if (RGB_DIR == RGB_down)
  {
    if (LED_RGB.Blue > wireRGB_MIN.Blue)
      LED_RGB.Blue--;
    else if (LED_RGB.Blue <= wireRGB_MIN.Blue)
    {
      if (LED_RGB.Green > wireRGB_MIN.Green)
        LED_RGB.Green--;
      else if (LED_RGB.Green <= wireRGB_MIN.Green)
      {
        if (LED_RGB.Red > wireRGB_MIN.Red)
          LED_RGB.Red--;
        else if (LED_RGB.Red <= wireRGB_MIN.Red)
        {
          RGB_DIR = RGB_up;
          wireRGB_MAX.Red = random(0, 255);
          wireRGB_MAX.Green = random(0, 255);
          wireRGB_MAX.Blue = random(0, 255);
        }
      }
    }
    //Serial.println(String("RGB_DIR=") + String(RGB_DIR) + String(", LED_RGB.Red=") + String(LED_RGB.Red) + String(",LED_RGB.Green=") + String(LED_RGB.Green) + String(",LED_RGB.Blue=") + String(LED_RGB.Blue));
  }
  else if (RGB_DIR == RGB_up)
  {
    if (LED_RGB.Red < wireRGB_MAX.Red)
      LED_RGB.Red++;
    else if (LED_RGB.Red >= wireRGB_MAX.Red)
    {
      if (LED_RGB.Green < wireRGB_MAX.Green)
        LED_RGB.Green++;
      else if (LED_RGB.Green >= wireRGB_MAX.Green)
      {
        if (LED_RGB.Blue < wireRGB_MAX.Blue)
          LED_RGB.Blue++;
        else if (LED_RGB.Blue >= wireRGB_MAX.Blue)
        {
          RGB_DIR = RGB_down;
          wireRGB_MIN.Red = random(0, 255);
          wireRGB_MIN.Green = random(0, 255);
          wireRGB_MIN.Blue = random(0, 255);
        }
      }
    }
    //Serial.println(String("RGB_DIR=") + String(RGB_DIR) + String(", LED_RGB.Red=") + String(LED_RGB.Red) + String(",LED_RGB.Green=") + String(LED_RGB.Green) + String(",LED_RGB.Blue=") + String(LED_RGB.Blue));
  }

  WireLED_color_SET(&LED_RGB);
  delay(5);
}


void setup() {
  // put your setup code here, to run once:
  uint8_t i;
  /*********任务指针初始化*****************************************/
  Current_PROM_Mode = Ges_cycLED;
  Prom_Start_Time.Program_name = Current_PROM_Mode;
  Prom_Start_Time.Start_Time = 0;
  Serial.println(String("Current_PROM_Mode:") + String(Current_PROM_Mode));
  /*********定义电机引脚*****************************************/
  pinMode(IA1, OUTPUT);
  pinMode(IA2, OUTPUT);
  pinMode(IB1, OUTPUT);
  pinMode(IB2, OUTPUT);
  /************************************************************/
  /************************************************************/
  /*********定义灯带引脚和初始状态********************************/
  neoPixel_2.begin(PINS_wireLED, 9);//灯带引脚和小灯数量
  LED_RGB.Red = 0x00;
  LED_RGB.Green = 0x00;
  LED_RGB.Blue = 0x00;
  WireLED_color_SET(&LED_RGB);
  wireRGB_MIN.Red = 0;
  wireRGB_MIN.Green = 0;
  wireRGB_MIN.Blue = 0;
  wireRGB_MAX.Red = 0xff;
  wireRGB_MAX.Green = 0xff;
  wireRGB_MAX.Blue = 0xff;
  RGB_DIR = RGB_up;
  for (i = 0; i < 10; i++)
    MotionWithGeature[i] = 0;
  /************************************************************/
  Serial.begin(9600); //初始化串口
  /************************************************************/
  /********初始化手势识别芯片************************************/
  attachInterrupt(0, interruptRoutine, FALLING);
  if ( apds.init() ) {//i2c引脚配置和初始值设定
    Serial.println(F("APDS-9960 initialization complete"));
  } else {
    Serial.println(F("Something went wrong during APDS-9960 init!"));
  }
  if ( apds.enableGestureSensor(true) ) {//使能手势识别
    Serial.println(F("Gesture sensor is now running"));
  } else {
    Serial.println(F("Something went wrong during gesture sensor init!"));
  }
  Gesture_command = 0;
  /************************************************************/
}

void loop() {
  uint8_t i;
  switch (Current_PROM_Mode) {
    case Ges_cycLED:
      WireLED_RGBloop();
      if (Gesture_command == Far_gesture)
      {
        Gesture_command = No_gesture;
        while (!apds.disableGestureSensor())
          Serial.println("Disabling Gesture Sensor");
        if (apds.enableLightSensor(false))
          Serial.println("Proximity Sensor enabled");
        Current_PROM_Mode = Lit_detect;
        Prom_Start_Time.Program_name = Current_PROM_Mode;
        Prom_Start_Time.Start_Time = millis();//记录程序开始时的系统时间
      }
      else if (Gesture_command == Down_gesture) {
        Current_PROM_Mode = Ges_staLED;
        Gesture_command = No_gesture;
      } break;

    case Ges_staLED:
      WireLED_color_SET(&LED_RGB);
      if (Gesture_command == Down_gesture) {
        Gesture_command = No_gesture;
        Current_PROM_Mode = Ges_cycLED;
      }
      else if (Gesture_command == Far_gesture) {
        Car_action(Turn_left);
        delay(100);
        Car_action(Turn_right);
        delay(100);
        Car_action(Stop);
        for (i = 0; i < 10; i++)
          MotionWithGeature[i] = 0;
        MWG_point = 0;
        Current_PROM_Mode = Ges_motionMEM;
        Prom_Start_Time.Program_name = Current_PROM_Mode;
        Prom_Start_Time.Start_Time = millis();//记录程序开始时的系统时间
      } break;

    case Lit_detect:
      if (  !apds.readAmbientLight(Detected_Color.Ambient) ||
            !apds.readRedLight(Detected_Color.Red) ||
            !apds.readGreenLight(Detected_Color.Green) ||
            !apds.readBlueLight(Detected_Color.Blue) ) {
        Serial.println("Error reading light values");
      } else {
        LED_RGB.Red = Detected_Color.Red * 255 * 3 / 100;
        LED_RGB.Green = Detected_Color.Green * 255 * 3 / 100;
        LED_RGB.Blue = Detected_Color.Blue * 255 * 3 / 100;
        WireLED_color_SET(&LED_RGB);
      }
      if ((millis() - Prom_Start_Time.Start_Time) >= 8000) //超时退出颜色识别（8秒）
      {
        while (!apds.disableLightSensor())
          Serial.println("Disabling Color Sensor");
        Serial.println("Color Sensor Disabled");
        while (!apds.enableGestureSensor(true))
          Serial.println("Enabling Gesture Sensor ");
        Serial.println("Gesture Sensor enabled");
        Gesture_command = 0;
        Current_PROM_Mode = Ges_staLED;
        Prom_Start_Time.Program_name = Current_PROM_Mode;
      } break;

    case Ges_motionMEM:
      if (isr_flag) {
        isr_flag = 0;
        gesture_serial_test();
        if ((Gesture_command == Up_gesture || Gesture_command == Down_gesture
             || Gesture_command == Right_gesture || Gesture_command == Left_gesture)) {
          Serial.println(F("Ges_motionMEM"));
          LED_RGB_MEM.Red = 255;
          LED_RGB_MEM.Green = 255;
          LED_RGB_MEM.Blue = 255;
          WireLED_color_SET(&LED_RGB_MEM);
          delay(100);
          WireLED_color_SET(&LED_RGB);
          MotionWithGeature[MWG_point] = Gesture_command;
          Gesture_command = No_gesture;
          MWG_point++;
        }
      }
      if ((millis() - Prom_Start_Time.Start_Time) >= 10000 || MWG_point == 10)
      { Current_PROM_Mode = Motion_busy;
        MWG_point = 0;
      }
      break;

    case Motion_busy:
      WireLED_RGBloop();
      switch (MotionWithGeature[MWG_point]) {
        case Right_gesture:
          Car_action(Turn_left);
          delay(700);
          break;
        case Left_gesture:
          Car_action(Turn_right);
          delay(700);
          break;
        case Up_gesture:
          Car_action(Back);
          delay(500);
          break;
        case Down_gesture:
          Car_action(Forward);
          delay(500);
          break;
      }
      MWG_point++;
      if ((MotionWithGeature[MWG_point] == 0) || MWG_point == 10)
      {
        Car_action(Stop);
        Current_PROM_Mode = Ges_cycLED;
      }
  }

  if ( isr_flag == 1 && (Current_PROM_Mode == Ges_cycLED || Current_PROM_Mode == Ges_staLED))
  {
    Serial.println(F("Gesture interrupt"));
    isr_flag = 0;
    gesture_serial_test();
    Serial.println(String("Gesture_command") + String(Gesture_command));
  }
}
void interruptRoutine() {
  isr_flag = 1;

}
void gesture_serial_test()
{
  if ( apds.isGestureAvailable() ) {
    switch ( apds.readGesture() ) {

      case Down_gesture:
        Serial.println("DOWN");
        Gesture_command = Down_gesture;
        break;
      case Up_gesture:
        Serial.println("UP");
        Gesture_command = Up_gesture;
        if (Current_PROM_Mode == Ges_cycLED || Current_PROM_Mode == Ges_staLED) {
          LED_RGB.Red = random(0, 255);
          LED_RGB.Green = random(0, 255);
          LED_RGB.Blue = random(0, 255);
          WireLED_color_SET(&LED_RGB);
        }
        break;
      case Right_gesture:
        Serial.println("RIGHT");
        Gesture_command = Right_gesture;
        if (Current_PROM_Mode == Ges_cycLED || Current_PROM_Mode == Ges_staLED) {
          LED_RGB.Red = 0xff;
          WireLED_color_SET(&LED_RGB);
          Car_action(Turn_left);
          delay(900);
          Car_action(Stop);
        }
        break;
      case Left_gesture:
        Serial.println("LEFT");
        Gesture_command = Left_gesture;
        if (Current_PROM_Mode == Ges_cycLED || Current_PROM_Mode == Ges_staLED) {
          LED_RGB.Blue = 0xff;
          WireLED_color_SET(&LED_RGB);
          Car_action(Turn_right);
          delay(900);
          Car_action(Stop);
        }
        break;
      case Near_gesture:
        Serial.println("NEAR");
        Gesture_command = Near_gesture;
        break;
      case Far_gesture:
        Serial.println("FAR");
        Gesture_command = Far_gesture;
        neoPixel_2.setRangeColor(0, 7, 0x0000ff);
        break;
      default:
        Serial.println("NONE");
        Gesture_command = No_gesture;
        if (Current_PROM_Mode == Ges_cycLED || Current_PROM_Mode == Ges_staLED) {
          LED_RGB.Red = random(0, 255);
          LED_RGB.Green = random(0, 255);
          LED_RGB.Blue = random(0, 255);
          WireLED_color_SET(&LED_RGB);
        }
    }
  }
}
